#ifndef _MLIB_H_
#define _MLIB_H_
#include <tigcclib.h>
#include <graph.h>
#include <estack.h>
#include <args.h>

// Sentinel value for variable argument functions
#define LASTARG -1

// The matrix operations
enum MatrixOps {MOP_MULT = 0x00, MOP_DIV = 0x01, MOP_ADD = 0x02, MOP_SUBTR = 0x03};
// Alignment values when printing data into fixed spaces ( see prAlnStr() and prAlnColVecAt() )
enum Alignments {AL_LEFT = 0x00, AL_RIGHT = 0x01, AL_CENTER = 0x03, AL_DECIMAL = 0x04};
typedef unsigned short ushort;

// Matrix struct
typedef struct {
	ushort m;			// number of rows
	ushort n;			// number of columns
	char **specials;	// an array of strings that mirrors the numerical elements
	float *dat;			// the actual elements
} Matrix;

void **allocptrs=NULL;		// List of pointers allocated

/* GetMatArg - Get a matrix argument from the stack
 *  aptr	stack pointer
 *	mat		pointer to pointer to hold Matrix
 */
void GetMatArg(ESI *aptr, Matrix **mat);

/* PrintMatrix - Print a matrix
 *  mat		matrix to print
 *	fw		field width for each element (OPTIONAL) default=5
 *	prec	precision for each element (number of decimal places) (OPTIONAL) default=2
 *	zpad	zero padding flag (0=>pad with spaces, 1=>pad with zeroes) (OPTIONAL) default =0
 * Less than four arguments requires LASTARG as the last argument
 */
void printMatrix(Matrix *mat, ...);

/* prColVecAt - Print a column vector at a given location on the screen
 *	delim	0: no delimiter, 1: print '|' around the matrix
 *	x,y		pixel location on screen (0,0 is top left)
 *	mat		matrix to print
 *	col		column to print if mat is not a column vector (required if mat is rectangular)
 */
void prColVecAt(ushort delim, ushort x, ushort y, Matrix *mat, ...);

/* prAlnColVecAt - Print aligned column vector at a given location on the screen
 *	delim	0: no delimiter, 1: print '|' around the matrix
 *	colPW	column width in pixels
 *	align	Alignment of output within colPW  (AL_LEFT, AL_RIGHT, AL_CENTER, or AL_DECIMAL)
 *	x,y		pixel location on screen (0,0 is top left)
 *	mat		matrix to print
 *	col		column to print if mat is not a column vector (required if mat is rectangular)
 */
void prAlnColVecAt(ushort delim, ushort colPW, ushort align, ushort x, ushort y, Matrix *mat, ...);

/* prFmtVecAt - Print formatted column vector at a given location with label and rules
 *	x,y		  pixel location on screen (0,0 is top left)
 *	label	  label at top of column
 *	colWidth  column width in characters
 *	footer	  optional extra footer data to add as extra entry before bottom rule
 */
void prFmtVecAt(ushort x, ushort y, Matrix *mat, char *label, ushort colWidth, float *footer);

/* prAlnStr - print aligned string within a fixed width cell
 *	x,y		pixel location on screen (0,0 is top left)
 *	cellPW	cell pixel width (NOT characters!)
 *	align	alignment to use within cell (AL_LEFT, AL_RIGHT, AL_CENTER, or AL_DECIMAL)
 *	str		string to print
 *	tChar	char to put at the end of truncated strings (e.g. 3.1415926 truncated to 8 digits
 *			with '#' becomes '3.14159#'
 */
void prAlnStr(ushort x, ushort y, ushort cellPW, ushort align, char *str, char tChar);

/* newMat - Create a new matrix
 *	m		number of rows
 *	n		number of columns
 *	ddat	default data value for all cells
 * NOTE: 1. The data array is linear, and individual elements of Matrix *x would be accessed with
 *			something like x->dat[i*x->n + j]
 *		 2. One extra entry is allocated to store miscellaneous data
 */
Matrix *newMat(ushort m, ushort n, float ddat);


/* All of these matrix functions return pointers to newly created matrices and leave their
 * arguments intact
 */

/* eWiseOp - Elementwise operations: perform op between corresponding elements of matrices
 *				of identical dimension
 *	mat1	first matrix
 *	op		operation to perform (MOP_MULT, MOP_DIV, MOP_ADD, or MOP_SUBTR)
 *	mat2	second matrix
 */
Matrix *eWiseOp(Matrix *mat1, ushort op, Matrix *mat2);

/* sOpMat - Scalar operation on amatrix
 *	x		scalar value
 *	op		operation to perform (MOP_MULT, MOP_DIV, MOP_ADD, or MOP_SUBTR)
 *	mat		matrix to operate on (order is always where, for Mij (the ij'th element),
 *			the result value is x?Mij (matters for division and subtraction)
 */
Matrix *sOpMat(float x, ushort op, Matrix *mat);

/* colSum - creates a row vector of the column sums of a matrix
 *	mat		matrix to perform column sum on
 */
Matrix *colSum(Matrix *mat);

/* colMax - creates a row vector of the maxima of the columns of a matrix
 *	mat		matrix to perform max on
 */
Matrix *colMax(Matrix *mat);

/* trn - transpose of input matrix
 *	mat		matrix to transpose
 */
Matrix *trn(Matrix *mat);

/* InputStr - take keyboard input string termined by ENTER
 *	buffer	allocated string buffer to store output (existing data is printed and used as default entry)
 *	maxlen	maximum length of string to accept
 */
void InputStr(char *buffer, ushort maxlen);

/* smalloc - smart memory allocation-- allocate memory and keep track of allocated block for later release
 *	len		number of bytes
 */
void *smalloc(unsigned long);

/* sfree - smart memory free-- free memory previously allocated with smalloc()
 *	ptr		pointer to memory block to free
 * NOTE: DO NOT free any memory allocated with smalloc() with the free() comment, or later use of freeAP() will cause a crash
 */
void sfree(void *);

/* freeAP() - smart free allocated pointer list-- free all memory allocated with smalloc() since start of program
 */
void freeAP();

#endif